<?php namespace Notes\Breadcrumb;

use Notes\ObjectReflection,
    Notes\ObjectResolver;

use Notes\Route\RouteFetcher;
use Psr\Http\Message\ServerRequestInterface;
use Psr\SimpleCache\CacheInterface;
use RuntimeException, DirectoryIterator, Generator;

class Breadcrumb extends RouteFetcher {

    public function getRouteTree(\Notes\Route\Annotation\Method\Route $route) : array
    {
        $tree = [];

        $routes = $this->compile([
            'object' => [ Annotation\Object\Route::class, Attribute\Object\Route::class ],
            'method' => [ Annotation\Method\Route::class ],
        ]);

        $crumbs = $this->compile([
            'method' => [ Annotation\Method\Breadcrumb::class, Attribute\Method\Breadcrumb::class ],
        ]);

        while( $route ) {
            $crumb = $this->getBreadcrumbAnnotation($crumbs, $route);

            if ($crumb) {
                empty($tree) && $crumb->current = true;

                $tree[$route->name] = $crumb;

                $route = $this->getRouteAnnotation($routes, $crumb);
            }
            else {
                $route = null;
            }
        };

        return array_reverse($tree);
    }

    protected function getRouteAnnotation(array $list, Annotation\Method\Breadcrumb $crumb) : null|\Notes\Route\Annotation\Method\Route
    {
        if ($crumb->parent ?? null) {
            foreach ($list as $route) {
                if ($crumb->parent === $route->name) {
                    return $route;
                }
            }
        }

        return null;
    }

    protected function getBreadcrumbAnnotation(array $list, \Notes\Route\Annotation\Method\Route $route) : null|Annotation\Method\Breadcrumb
    {
        foreach($list as $crumb) {
            if ($crumb->class === $route->class && $crumb->classMethod === $route->classMethod) {
                $crumb->routeAnnotation = $route;

                return $crumb;
            }
        }

        return null;
    }
}
